/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License") you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code in this file was released on September 2, 2004
 * 
 * Unless otherwise stated, the Initial Developer of the
 * Original Code is David Perry.  Portions created by David Perry are
 * Copyright (C) 2002 David Perry.  All rights reserved.
 * 
 * Contributor(s):
 *   David Perry <d.perry@utoronto.ca> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable
 * instead of those above.  If you wish to allow use of your
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

/*
 * Global variables
 */

var gAuwrEditor;        // Editor object (?? see below)
var gAuWrapped;         // Flag - have we wrapped already?

/*
 * autoRewrapLoad() - window onload handler
 */
function autoRewrapLoad() {
    dump( "autoRewrapLoaded()\n" );

    var compStateListener = new auwrComposeStateListener();
    gMsgCompose.RegisterStateListener(compStateListener);
}

/*
 * autoRewrapReload() - window compose-window-reload handler
 */
function autoRewrapReload() {
    gAuWrapped = false;
}

/*
 * autoRewrap() - do the dirty deed
 */
function autoRewrap() {
    dump( "autoRewrap()\n" );
    gAuWrapped = true;
    try {
        if(window.GetCurrentEditor)
        {
            var ed = window.GetCurrentEditor();

            /* We want to re-select the current selection after we
                mess with the document.  First we need to save it */
            ed.beginTransaction();
            var selection = ed.selection;
            var ranges = [];
            if( selection && selection.rangeCount )
            {
                    for( var i=0; i < selection.rangeCount; i++ )
                    {
                            var r = selection.getRangeAt(i);
                            ranges.push( { range:r, start: r.startOffset, end:r.endOffset, collapsed: r.collapsed } );
                    }
            }

            ed.rewrap(true);

            /* Restore the last selection (which may just be the zero-size
                 selection referring to the cursor's location */
            while( selection && ranges.length )
            {
                    var r = ranges.shift();
                    if( !r.collapsed )
                            r.range.setEnd( r.range.endContainer, r.end );
                    selection.addRange( r.range );
                    if( r.collapsed )
                            selection.collapse( r.range.startContainer, r.start );
            }
            // And we're done
            ed.endTransaction();
        }
        else {
            dump("autoRewrap: Couldn't get current editor.  Bummer.\n");
        }
    }
    catch(ex) {
        dump("autoRewrap exception: "+ex+"\n");
        ed.endTransaction();
        ed.undo(1); // undo any damage done
        gAuWrapped = false;
    }
    return;
}

/*
 * The listeners that do the magic
 * -------------------------------
 *
 * This mechanism was observed in Enigmail (http://enigmail.mozdev.org).
 * My understanding of it is a bit shaky, but hey, it seems to work.
 * The first listener gets attached onload, and then it attaches the second,
 * which gets called when the document is editable.
 */

function auwrComposeStateListener() {
}

auwrComposeStateListener.prototype = {
    NotifyComposeFieldsReady : function() {
        dump("auwrComposeStateListener.NotifyComposeFieldsReady()\n");
        try {
            gAuwrEditor = gMsgCompose.editor.QueryInterface(Components.interfaces.nsIEditor);
        }
        catch(ex){}
        if( !gAuwrEditor ) return;

        var docStateListener = new auwrDocStateListener();
        gAuwrEditor.addDocumentStateListener(docStateListener);
    }
};

function auwrDocStateListener() {
}

auwrDocStateListener.prototype = {
    QueryInterface: function (iid) {
        if (!iid.equals(Components.interfaces.nsIDocumentStateListener) &&
            !iid.equals(Components.interfaces.nsISupports))
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    },

    NotifyDocumentStateChanged: function(nowDirty) {
        dump("auwrDocStateListener.NotifyDocumentStateChanged("+nowDirty+"): ");
        var isEmpty = gAuwrEditor.documentIsEmpty;
        var isEditable = gAuwrEditor.isDocumentEditable;
        dump( (isEmpty?"empty":"not empty")+", "
            +(isEditable?"editable":"not editable")+"\n" );

        if( isEmpty || !isEditable )
            return;

        if( !gAuWrapped )
            setTimeout(autoRewrap, 100);
    }
}

/* EOF */
